﻿/*
 * Copyright 2010 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;

namespace Vajhoej.Record.Test
{
    [Struct]
    public class Data
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv;
        [StructField(N=1,Type=FieldType.FP8)]
        private double xv;
        [StructField(N=2,Type=FieldType.FIXSTR,Length=8,Encoding="ISO-8859-1")]
        private String sv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
        public double Xv
        {
            get { return xv; }
            set { xv = value; }
        }
        public string Sv
        {
            get { return sv; }
            set { sv = value; }
        }
    }
    [Struct(Alignment=Alignment.PACKED)]
    public class AlignData0
    {
        [StructField(N=0,Type=FieldType.INT1)]
        private sbyte i1;
        [StructField(N=1,Type=FieldType.INT2)]
        private short i2;
        [StructField(N=2,Type=FieldType.INT4)]
        private int i4;
        [StructField(N=3,Type=FieldType.INT8)]
        private long i8;
        public sbyte I1
        {
            get { return i1; }
            set { i1 = value; }
        }
        public short I2
        {
            get { return i2; }
            set { i2 = value; }
        }
        public int I4
        {
            get { return i4; }
            set { i4 = value; }
        }
        public long I8
        {
            get { return i8; }
            set { i8 = value; }
        }
    }
    [Struct(Alignment=Alignment.NATURAL)]
    public class AlignData1
    {
        [StructField(N=0,Type=FieldType.INT1)]
        private sbyte i1;
        [StructField(N=1,Type=FieldType.INT2)]
        private short i2;
        [StructField(N=2,Type=FieldType.INT4)]
        private int i4;
        [StructField(N=3,Type=FieldType.INT8)]
        private long i8;
        public sbyte I1
        {
            get { return i1; }
            set { i1 = value; }
        }
        public short I2
        {
            get { return i2; }
            set { i2 = value; }
        }
        public int I4
        {
            get { return i4; }
            set { i4 = value; }
        }
        public long I8
        {
            get { return i8; }
            set { i8 = value; }
        }
    }
    [Struct(Alignment=Alignment.ALIGN8)]
    public class AlignData2
    {
        [StructField(N=0,Type=FieldType.INT1)]
        private sbyte i1;
        [StructField(N=1,Type=FieldType.INT2)]
        private short i2;
        [StructField(N=2,Type=FieldType.INT4)]
        private int i4;
        [StructField(N=3,Type=FieldType.INT8)]
        private long i8;
        public sbyte I1
        {
            get { return i1; }
            set { i1 = value; }
        }
        public short I2
        {
            get { return i2; }
            set { i2 = value; }
        }
        public int I4
        {
            get { return i4; }
            set { i4 = value; }
        }
        public long I8
        {
            get { return i8; }
            set { i8 = value; }
        }
    }
    [Struct(Alignment=Alignment.ALIGN4,Endpad=false)]
    public class EndPadData1
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv;
        [StructField(N=1,Type=FieldType.INT1)]
        private sbyte bv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
        public sbyte Bv
        {
            get { return bv; }
            set { bv = value; }
        }
    }
    [Struct(Alignment=Alignment.ALIGN4,Endpad=true)]
    public class EndPadData2
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv;
        [StructField(N=1,Type=FieldType.INT1)]
        private sbyte bv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
        public sbyte Bv
        {
            get { return bv; }
            set { bv = value; }
        }
    }
    [Struct(Alignment=Alignment.NATURAL,Endpad=true)]
    public class EndPadData3
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv;
        [StructField(N=1,Type=FieldType.FP8)]
        private double xv;
        [StructField(N=2,Type=FieldType.INT1)]
        private sbyte bv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
        public double Xv
        {
            get { return xv; }
            set { xv = value; }
        }
        public sbyte Bv
        {
            get { return bv; }
            set { bv = value; }
        }
    }
    [Struct]
    public class SuperData
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int id;
        [StructField(N=1,Type=FieldType.INT4)]
        [Selector]
        [SubType(Value=1,Type=typeof(SubDataOne))]
        [SubType(Value=2,Type=typeof(SubDataTwo))]
        private int typ;
        public int Id
        {
            get { return id; }
            set { id = value; }
        }
        public int Typ
        {
            get { return typ; }
            set { typ = value; }
        }
    }
    [Struct]
    public class SubDataOne : SuperData
    {
        [StructField(N=2,Type=FieldType.FP8)]
        private double x;
        public double X
        {
            get { return x; }
            set { x = value; }
        }
    }
    [Struct]
    public class SubDataTwo : SuperData
    {
        [StructField(N=2,Type=FieldType.FIXSTR,Length=12,Encoding="ISO-8859-1")]
        private String s;
        public string S
        {
            get { return s; }
            set { s = value; }
        }
    }
    [Struct]
    public class SuperDataPad
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int id;
        [StructField(N=1,Type=FieldType.INT4)]
        [Selector(Pad=true)]
        [SubType(Value=1,Type=typeof(SubDataOnePad))]
        [SubType(Value=2,Type=typeof(SubDataTwoPad))]
        private int typ;
        public int Id
        {
            get { return id; }
            set { id = value; }
        }
        public int Typ
        {
            get { return typ; }
            set { typ = value; }
        }
    }
    [Struct]
    public class SubDataOnePad : SuperDataPad
    {
        [StructField(N=2,Type=FieldType.FP8)]
        private double x;
        public double X
        {
            get { return x; }
            set { x = value; }
        }
    }
    [Struct]
    public class SubDataTwoPad : SuperDataPad
    {
        [StructField(N=2,Type=FieldType.FIXSTR,Length=12,Encoding="ISO-8859-1")]
        private String s;
        public string S
        {
            get { return s; }
            set { s = value; }
        }
    }
    [Struct]
    public class MainData
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int i1;
        [StructField(N=1,Type=FieldType.INT4)]
        private int i2;
        [StructField(N=2,Type=FieldType.STRUCT)]
        public FieldData s;
        [StructField(N=3,Type=FieldType.INT4)]
        private int i5;
        public int I1
        {
            get { return i1; }
            set { i1 = value; }
        }
        public int I2
        {
            get { return i2; }
            set { i2 = value; }
        }
        public FieldData S
        {
            get { return s; }
            set { s = value; }
        }
        public int I5
        {
            get { return i5; }
            set { i5 = value; }
        }
    }
    [Struct]
    public class FieldData
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int i3;
        [StructField(N=1,Type=FieldType.INT4)]
        private int i4;
        public int I3
        {
            get { return i3; }
            set { i3 = value; }
        }
        public int I4
        {
            get { return i4; }
            set { i4 = value; }
        }
    }
    [Struct]
    public class BooleanData
    {
        [StructField(N=0,Type=FieldType.BOOLEAN,Length=4)]
        private bool b1;
        [StructField(N=1,Type=FieldType.BOOLEAN,Length=2)]
        private bool b2;
        [StructField(N=2,Type=FieldType.BOOLEAN,Length=1)]
        private bool b3;
        public bool B1
        {
            get { return b1; }
            set { b1 = value; }
        }
        public bool B2
        {
            get { return b2; }
            set { b2 = value; }
        }
        public bool B3
        {
            get { return b3; }
            set { b3 = value; }
        }
    }
    [Struct]
    public class BitData
    {
        [StructField(N=0,Type=FieldType.INT1)]
        private int i1;
        [StructField(N=1,Type=FieldType.BIT,Length=4)]
        private int n1;
        [StructField(N=2,Type=FieldType.BIT,Length=4)]
        private int n2;
        [StructField(N=3,Type=FieldType.INT1)]
        private int i2;
        [StructField(N=4,Type=FieldType.BIT,Length=1)]
        private int z1;
        [StructField(N=5,Type=FieldType.BIT,Length=15)]
        private int z2;
        public int I1
        {
            get { return i1; }
            set { i1 = value; }
        }
        public int N1
        {
            get { return n1; }
            set { n1 = value; }
        }
        public int N2
        {
            get { return n2; }
            set { n2 = value; }
        }
        public int I2
        {
            get { return i2; }
            set { i2 = value; }
        }
        public int Z1
        {
            get { return z1; }
            set { z1 = value; }
        }
        public int Z2
        {
            get { return z2; }
            set { z2 = value; }
        }
    }
    [Struct]
    public class StringData
    {
        [StructField(N=0,Type=FieldType.FIXSTR,Length=4,Encoding="ISO-8859-1")]
        private String s1;
        [StructField(N=1,Type=FieldType.VARSTR,Encoding="ISO-8859-1")]
        private String s2;
        [StructField(N=2,Type=FieldType.VARFIXSTR,Length=4,Encoding="ISO-8859-1")]
        private String s3;
        [StructField(N=3,Type=FieldType.FIXSTRNULTERM,Length=4,Encoding="ISO-8859-1")]
        private String s4;
        public string S1
        {
            get { return s1; }
            set { s1 = value; }
        }
        public string S2
        {
            get { return s2; }
            set { s2 = value; }
        }
        public string S3
        {
            get { return s3; }
            set { s3 = value; }
        }
        public string S4
        {
            get { return s4; }
            set { s4 = value; }
        }
    }
    [Struct]
    public class UnsignedData
    {
        [StructField(N=0,Type=FieldType.UINT1)]
        private byte ui1v;
        [StructField(N=1,Type=FieldType.UINT2)]
        private ushort ui2v;
        [StructField(N=2,Type=FieldType.UINT4)]
        private uint ui4v;
        public byte Ui1v
        {
            get { return ui1v; }
            set { ui1v = value; }
        }
        public ushort Ui2v
        {
            get { return ui2v; }
            set { ui2v = value; }
        }
        public uint Ui4v
        {
            get { return ui4v; }
            set { ui4v = value; }
        }
    }
    [Struct]
    public class VariableData
    {
        [StructField(N=0,Type=FieldType.FIXSTR,Length=0)]
        private String s1;
        [StructField(N=1,Type=FieldType.FIXSTR,Length=0)]
        private String s2;
        [StructField(N=2,Type=FieldType.FIXSTR,Length=0)]
        private String s3;
        [StructField(N=3,Type=FieldType.FIXSTR,Length=0)]
        private String s4;
        public string S1
        {
            get { return s1; }
            set { s1 = value; }
        }
        public string S2
        {
            get { return s2; }
            set { s2 = value; }
        }
        public string S3
        {
            get { return s3; }
            set { s3 = value; }
        }
        public string S4
        {
            get { return s4; }
            set { s4 = value; }
        }
    }
    [Struct(Endianess=Endian.BIG)]
    public class BigEndianData {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
    }
    [Struct]
    public class BCDData
    {
        [StructField(N=0,Type=FieldType.PACKEDBCD,Length=4,Decimals=2)]
        private decimal v1;
        [StructField(N=1,Type=FieldType.ZONEDBCD,Length=6,Decimals=2)]
        private decimal v2;
        public decimal V1
        {
            get { return v1; }
            set { v1 = value; }
        }
        public decimal V2
        {
            get { return v2; }
            set { v2 = value; }
        }
    }
    [Struct]
    public class VaxFloatData
    {
        [StructField(N=0,Type=FieldType.VAXFP4)]
        private float v1;
        [StructField(N=1,Type=FieldType.VAXFP8)]
        private double v2;
        public float V1
        {
            get { return v1; }
            set { v1 = value; }
        }
        public double V2
        {
            get { return v2; }
            set { v2 = value; }
        }
    }
    [Struct]
    public class TimeData
    {
        [StructField(N=0,Type=FieldType.JAVATIME)]
        private DateTime t1;
        [StructField(N=1,Type=FieldType.VMSTIME)]
        private DateTime t2;
        [StructField(N=2,Type=FieldType.UNIXTIME)]
        private DateTime t3;
        public DateTime T1
        {
            get { return t1; }
            set { t1 = value; }
        }
        public DateTime T2
        {
            get { return t2; }
            set { t2 = value; }
        }
        public DateTime T3
        {
            get { return t3; }
            set { t3 = value; }
        }
    }
    [Struct]
    public class ErrorData1
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private String sv;
        public string Sv
        {
            get { return sv; }
            set { sv = value; }
        }
    }
    [Struct]
    public class ErrorData2
    {
        [StructField(N=1,Type=FieldType.INT4)]
        private int iv;
        public int Iv
        {
            get { return iv; }
            set { iv = value; }
        }
    }
    [Struct]
    public class ErrorData3
    {
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv1;
        [StructField(N=0,Type=FieldType.INT4)]
        private int iv2;
        public int Iv1
        {
            get { return iv1; }
            set { iv1 = value; }
        }
        public int Iv2
        {
            get { return iv2; }
            set { iv2 = value; }
        }
    }
    [Struct]
    public class ArrayData {
        [ArrayField(Elements=3)]
        [StructField(N=0,Type=FieldType.INT4)]
        private int[] iv;
        [ArrayField(Elements=3)]
        [StructField(N=1,Type=FieldType.FP8)]
        private double[] xv;
        public ArrayData()
        {
            iv = new int[3];
            xv = new double[3];
        }
        public int GetIv(int ix)
        {
            return iv[ix];
        }
        public void SetIv(int ix, int iv)
        {
            this.iv[ix] = iv;
        }
        public double GetXv(int ix)
        {
            return xv[ix];
        }
        public void SetXv(int ix, double xv)
        {
            this.xv[ix] = xv;
        }
    }
    [Struct]
    public class ValueTypeData
    {
        [StructField(N=0,Type=FieldType.INT4)]
        public int Iv;
        [StructField(N=1,Type=FieldType.FP8)]
        public double Xv;
        [StructField(N=2,Type=FieldType.FIXSTR,Length=8,Encoding="ISO-8859-1")]
        public String Sv;
    }
}
